"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tokenQuantizedAmount = exports.amountToQuantizedAmount = exports.ethToEth8 = exports.weiToEth8 = exports.eth8ToEth = exports.eth8ToWei = exports.ethToken = exports.sleep = exports.formatError = exports.payloadId = void 0;
var units_1 = require("@ethersproject/units");
var imx_link_types_1 = require("@imtbl/imx-link-types");
var ethers_1 = require("ethers");
var function_1 = require("fp-ts/function");
var fp_1 = require("./fp");
function payloadId() {
    var datePart = new Date().getTime() * Math.pow(10, 3);
    var extraPart = Math.floor(Math.random() * Math.pow(10, 3));
    var id = datePart + extraPart;
    return id;
}
exports.payloadId = payloadId;
function formatError(err) {
    if (err.response) {
        return new Error(JSON.stringify(err.response.data));
    }
    else if (err.request) {
        return new Error("Errno: " + err.errno + " Errcode: " + err.code);
    }
    else {
        return new Error(err.message);
    }
}
exports.formatError = formatError;
function sleep(time) {
    return new Promise(function (resolve) {
        setTimeout(resolve, time);
    });
}
exports.sleep = sleep;
exports.ethToken = {
    type: imx_link_types_1.ETHTokenType.ETH,
    data: {
        decimals: 18,
    },
};
exports.eth8ToWei = function (amount) {
    return function_1.pipe(amount.mul(Math.pow(10, 8)), imx_link_types_1.PositiveBigNumber.decode, fp_1.valueOrThrow);
};
exports.eth8ToEth = function (amount) {
    return function_1.pipe(amount.mul(Math.pow(10, 8)), String, units_1.formatEther, imx_link_types_1.PositiveNumberStringC.decode, fp_1.valueOrThrow);
};
exports.weiToEth8 = function (amount) {
    return function_1.pipe(amount.div(Math.pow(10, 8)), imx_link_types_1.PositiveBigNumber.decode, fp_1.valueOrThrow);
};
exports.ethToEth8 = function (amount) {
    return function_1.pipe(units_1.parseEther(amount), imx_link_types_1.PositiveBigNumber.decode, fp_1.valueOrThrow, exports.weiToEth8);
};
exports.amountToQuantizedAmount = function (amount, quantum) {
    if (quantum === '1') {
        return amount;
    }
    return function_1.pipe(amount.div(ethers_1.BigNumber.from(quantum)), imx_link_types_1.PositiveBigNumber.decode, fp_1.valueOrThrow);
};
/**
 * NOTE: 'quantum' value is required for 'ERC-20' tokens,
 * and should be retreived from the '/tokens' API reponse.
 */
exports.tokenQuantizedAmount = function (token, amount, quantum) {
    if (quantum === void 0) { quantum = '1'; }
    switch (token.type) {
        case imx_link_types_1.ETHTokenType.ETH: {
            return exports.weiToEth8(amount);
        }
        case imx_link_types_1.ERC20TokenType.ERC20: {
            return exports.amountToQuantizedAmount(amount, quantum);
        }
        default: {
            return amount;
        }
    }
};
//# sourceMappingURL=utils.js.map